@@ -1,32 +1,46 @@
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
- * Copyright 1988-1997, Patrick Powell, San Diego, CA
- *	 papowell@astart.com
+ * Copyright 1988-1999, Patrick Powell, San Diego, CA
+ *     papowell@astart.com
  * See LICENSE for conditions of use.
  *
- ***************************************************************************
- * MODULE: krb_auth.c
- * PURPOSE: do kerberos authentication
- **************************************************************************/
+ ***************************************************************************/
+
+ static char *const _id =
+"$Id: krb5_auth.c,v 5.1 1999/09/12 21:32:39 papowell Exp papowell $";
 
-static char *const _id =
-"$Id: krb5_auth.c,v 1.8 1997/10/15 04:06:28 papowell Exp $";
 
 #include "lp.h"
-#include "krb5_auth.h"
 #include "fileopen.h"
-#include "killchild.h"
+#include "child.h"
+#include "getqueue.h"
+#include "krb5_auth.h"
+
+#if !defined(HAVE_KRB5_H)
+
+int server_krb5_auth( char *keytabfile, char *service, int sock,
+	char **auth, char *err, int errlen, char *file )
+{
+	plp_snprintf( err, errlen,
+	"kerberos authentication facilities not compiled" );
+	return(1);
+}
+int server_krb5_status( int sock, char *err, int errlen, char *file )
+{
+	return(1);
+}
+
+#else
 
-#if defined(HAVE_KRB5_H)
 #include <krb5.h>
 #include <com_err.h>
 
-krb5_error_code krb5_read_message 
+ extern krb5_error_code krb5_read_message 
 	KRB5_PROTOTYPE((krb5_context,
 		   krb5_pointer, 
 		   krb5_data *));
-krb5_error_code krb5_write_message 
+ extern krb5_error_code krb5_write_message 
 	KRB5_PROTOTYPE((krb5_context,
 		   krb5_pointer, 
 		   krb5_data *));
@@ -41,33 +55,37 @@ krb5_error_code krb5_write_message
  *   Note: there is a memory leak if authentication fails,  so this
  *   should not be done in the main or non-exiting process
  */
-int des_read( krb5_context context, krb5_encrypt_block *eblock,
+ extern int des_read( krb5_context context, krb5_encrypt_block *eblock,
 	int fd, char *buf, int len, char *err, int errlen );
-int des_write( krb5_context context, krb5_encrypt_block *eblock,
+ extern int des_write( krb5_context context, krb5_encrypt_block *eblock,
 	int fd, char *buf, int len, char *err, int errlen );
 
-int server_krb5_auth( char *keytabfile, char *service, int sock,
-	char *auth, int authlen, char *err, int errlen, char *file )
+ /* we make these statics */
+ static krb5_context context = 0;
+ static krb5_auth_context auth_context = 0;
+ static krb5_keytab keytab = 0;  /* Allow specification on command line */
+ static krb5_principal server = 0;
+ static krb5_ticket * ticket = 0;
+
+ int server_krb5_auth( char *keytabfile, char *service, int sock,
+	char **auth, char *err, int errlen, char *file )
 {
-	krb5_context context = 0;
-	krb5_auth_context auth_context = 0;
-	krb5_keytab keytab = 0;  /* Allow specification on command line */
-	krb5_principal server = 0;
-	krb5_ticket * ticket = 0;
-	char *cname = 0;
 	int retval = 0;
 	int fd = -1;
-	char buffer[SMALLBUFFER];
 	krb5_data   inbuf, outbuf;
+	struct stat statb;
+	int status;
+	char *cname = 0;
 
-	DEBUG0("server_krb5_auth: keytab '%s', service '%s', sock %d, file '%s'",
+	DEBUG1("server_krb5_auth: keytab '%s', service '%s', sock %d, file '%s'",
 		keytabfile, service, sock, file );
+	err[0] = 0;
 	if ((retval = krb5_init_context(&context))){
 		plp_snprintf( err, errlen, "%s '%s'",
 			"krb5_init_context failed - '%s' ", error_message(retval) );
 		goto done;
 	}
-	if( (retval = krb5_kt_resolve(context, keytabfile, &keytab) ) ){
+	if( keytab == 0 && (retval = krb5_kt_resolve(context, keytabfile, &keytab) ) ){
 		plp_snprintf( err, errlen,
 			"krb5_kt_resolve failed - file %s '%s'",
 			keytabfile,
@@ -81,16 +99,12 @@ int server_krb5_auth( char *keytabfile, char *service, int sock,
 		   service, error_message(retval));
 		goto done;
 	}
-	if(DEBUGL0){
-		if((retval = krb5_unparse_name(context, server, &cname))){
-			plp_snprintf( err, errlen,
-				"krb5_unparse_name failed: %s", error_message(retval));
-			goto done;
-		}
-		logDebug( "server_krb5_auth: server '%s'", cname );
-		free(cname);
-		cname = 0;
+	if((retval = krb5_unparse_name(context, server, &cname))){
+		plp_snprintf( err, errlen,
+			"krb5_unparse_name failed: %s", error_message(retval));
+		goto done;
 	}
+	DEBUG1("server_krb5_auth: server '%s'", cname );
 
 	if((retval = krb5_recvauth(context, &auth_context, (krb5_pointer)&sock,
 				   service , server, 
@@ -98,7 +112,7 @@ int server_krb5_auth( char *keytabfile, char *service, int sock,
 				   keytab,  /* default keytab is NULL */
 				   &ticket))){
 		plp_snprintf( err, errlen,
-		"krb5_recvauth failed '%s'", error_message(retval));
+		"krb5_recvauth '%s' failed '%s'", cname, error_message(retval));
 		goto done;
 	}
 
@@ -109,10 +123,9 @@ int server_krb5_auth( char *keytabfile, char *service, int sock,
 			"krb5_unparse_name failed: %s",
 			error_message(retval));
 		goto done;
-	} else {
-		strncpy( auth, cname, authlen );
-		DEBUG0( "server_krb5_auth: client '%s'", cname );
 	}
+	if( auth ) *auth = safestrdup( cname,__FILE__,__LINE__);
+	DEBUG1( "server_krb5_auth: client '%s'", cname );
     /* initialize the initial vector */
     if((retval = krb5_auth_con_initivector(context, auth_context))){
 		plp_snprintf( err, errlen,
@@ -134,112 +147,118 @@ int server_krb5_auth( char *keytabfile, char *service, int sock,
   
 	memset( &inbuf, 0, sizeof(inbuf) );
 	memset( &outbuf, 0, sizeof(outbuf) );
-	if( file ){
-		struct stat statb;
-		int status;
 
-		fd = Checkwrite( file, &statb, O_WRONLY|O_TRUNC, 1, 0 );
-		DEBUG0( "server_krb5_auth: opened '%s', fd %d", file, fd );
-		if( fd < 0 ){
+	fd = Checkwrite( file, &statb, O_WRONLY|O_TRUNC, 1, 0 );
+	DEBUG1( "server_krb5_auth: opened for write '%s', fd %d", file, fd );
+	if( fd < 0 ){
+		plp_snprintf( err, errlen,
+			"file open failed: %s", Errormsg(errno));
+		retval = 1;
+		goto done;
+	}
+	while( (retval = krb5_read_message(context,&sock,&inbuf)) == 0 ){
+		if(DEBUGL5){
+			char small[16];
+			memcpy(small,inbuf.data,sizeof(small)-1);
+			small[sizeof(small)-1] = 0;
+			logDebug( "server_krb5_auth: got %d, '%s'",
+				inbuf.length, small );
+		}
+		if((retval = krb5_rd_priv(context,auth_context,
+			&inbuf,&outbuf,NULL))){
 			plp_snprintf( err, errlen,
-				"file open failed: %s", Errormsg(errno));
+				"krb5_rd_safe failed: %s", error_message(retval));
 			retval = 1;
 			goto done;
 		}
-		while( (retval = krb5_read_message(context,&sock,&inbuf)) == 0 ){
-			if(DEBUGL3){
-				char small[32];
-				memcpy(small,inbuf.data,sizeof(small)-1);
-				small[sizeof(small)-1] = 0;
-				logDebug( "server_krb5_auth: got %d, '%s'",
-					inbuf.length, small );
-			}
-			if((retval = krb5_rd_priv(context,auth_context,
-				&inbuf,&outbuf,NULL))){
-				plp_snprintf( err, errlen,
-					"krb5_rd_safe failed: %s", error_message(retval));
-				retval = 1;
-				goto done;
-			}
-			DEBUG4( "server_krb5_auth: read '%d': '%*s'",
-				outbuf.length, outbuf.length, outbuf.data );
-			status = Write_fd_len( fd, outbuf.data, outbuf.length );
-			if( status < 0 ){
-				plp_snprintf( err, errlen,
-					"write to file failed: %s", Errormsg(errno));
-				retval = 1;
-				goto done;
-			}
-			krb5_xfree(inbuf.data);
-			krb5_xfree(outbuf.data);
-			inbuf.length = 0;
-			outbuf.length = 0;
+		if( outbuf.data ) outbuf.data[outbuf.length] = 0;
+		status = Write_fd_len( fd, outbuf.data, outbuf.length );
+		if( status < 0 ){
+			plp_snprintf( err, errlen,
+				"write to file failed: %s", Errormsg(errno));
+			retval = 1;
+			goto done;
 		}
-		retval = 0;
-		close(fd);
-		fd = -1;
+		krb5_xfree(inbuf.data); inbuf.data = 0;
+		krb5_xfree(outbuf.data); outbuf.data = 0;
+		inbuf.length = 0;
+		outbuf.length = 0;
 	}
-	/* now we fork the other direction */
-	{
-		int fds[2];
-		int pid;
-		if( pipe(fds) < 0 ){
-			plp_snprintf( err, errlen, "pipe() failed - %s",
-				Errormsg(errno) );
-			retval = 1;
-			goto done;
-		}
-		if( (pid = dofork(0)) < 0 ){
-			plp_snprintf( err, errlen, "dofork() failed - %s",
-				Errormsg(errno) );
-			retval = 1;
-			goto done;
-		} else if( pid == 0 ){
-			/* we are the child */
-			close( fds[1] ); /* output side */
-			while( (retval = read( fds[0],buffer,sizeof(buffer)-1)) > 0 ){
-				inbuf.length = retval;
-				inbuf.data = buffer;
-				buffer[retval] = 0;
-				DEBUG4("server_krb5_auth: sending '%s'", buffer );
-				if((retval = krb5_mk_priv(context,auth_context,
-					&inbuf,&outbuf,NULL))){
-					plp_snprintf( err, errlen,
-						"krb5_mk_priv failed: %s", error_message(retval));
-					retval = 1;
-					Write_fd_str(2,err);
-					cleanup(0);
-				}
-				DEBUG4("server_krb5_auth: encoded length '%d'", outbuf.length );
-				if((retval= krb5_write_message(context,&sock,&outbuf))){
-					plp_snprintf( err, errlen,
-						"krb5_write_message failed: %s", error_message(retval));
-					Write_fd_str(2,err);
-					cleanup(0);
-				}
-				krb5_xfree(outbuf.data);
-			}
-			DEBUG0("server_krb5_auth: done", buffer );
-			cleanup(0);
-		} else {
-			close( fds[0] ); /* input side */
-			if( dup2( fds[1], sock ) == -1 ){
-				plp_snprintf( err, errlen, "dup2() failed - %s",
-					Errormsg(errno) );
-				retval = 1;
-				goto done;
-			}
-			close( fds[1] );
-		}
+	retval = 0;
+	close(fd); fd = -1;
+
+ done:
+	if( cname )		free(cname); cname = 0;
+	if( retval ){
+		if( fd >= 0 )	close(fd);
+		if( ticket )	krb5_free_ticket(context, ticket);
+		ticket = 0;
+		if( context && server )	krb5_free_principal(context, server);
+		server = 0;
+		if( context && auth_context)	krb5_auth_con_free(context, auth_context );
+		auth_context = 0;
+		if( context )	krb5_free_context(context);
+		context = 0;
 	}
-done:
+	DEBUG1( "server_krb5: retval %d, error: '%s'", retval, err );
+	return(retval);
+}
+
+
+int server_krb5_status( int sock, char *err, int errlen, char *file )
+{
+	int fd = -1;
+	int retval = 0;
+	struct stat statb;
+	char buffer[SMALLBUFFER];
+	krb5_data   inbuf, outbuf;
+
+	err[0] = 0;
+	memset( &inbuf, 0, sizeof(inbuf) );
+	memset( &outbuf, 0, sizeof(outbuf) );
+
+	fd = Checkread( file, &statb );
+	if( fd < 0 ){
+		plp_snprintf( err, errlen,
+			"file open failed: %s", Errormsg(errno));
+		retval = 1;
+		goto done;
+	}
+
+	while( (retval = read( fd,buffer,sizeof(buffer)-1)) > 0 ){
+		inbuf.length = retval;
+		inbuf.data = buffer;
+		buffer[retval] = 0;
+		DEBUG4("server_krb5_status: sending '%s'", buffer );
+		if((retval = krb5_mk_priv(context,auth_context,
+			&inbuf,&outbuf,NULL))){
+			plp_snprintf( err, errlen,
+				"krb5_mk_priv failed: %s", error_message(retval));
+			retval = 1;
+			goto done;
+		}
+		DEBUG4("server_krb5_status: encoded length '%d'", outbuf.length );
+		if((retval= krb5_write_message(context,&sock,&outbuf))){
+			plp_snprintf( err, errlen,
+				"krb5_write_message failed: %s", error_message(retval));
+			retval = 1;
+			goto done;
+		}
+		krb5_xfree(outbuf.data); outbuf.data = 0;
+	}
+	DEBUG1("server_krb5_status: done" );
+
+ done:
 	if( fd >= 0 )	close(fd);
-	if( cname )		free(cname);
 	if( ticket )	krb5_free_ticket(context, ticket);
-	if( server )	krb5_free_principal(context, server);
-	if( auth_context)	krb5_auth_con_free(context, auth_context );
+	ticket = 0;
+	if( context && server )	krb5_free_principal(context, server);
+	server = 0;
+	if( context && auth_context)	krb5_auth_con_free(context, auth_context );
+	auth_context = 0;
 	if( context )	krb5_free_context(context);
-	DEBUG0( "server_krb5: retval %d, error: '%s'", retval, err );
+	context = 0;
+	DEBUG1( "server_krb5_status: retval %d, error: '%s'", retval, err );
 	return(retval);
 }
 
@@ -260,7 +279,7 @@ done:
 #define VALIDATE 0
 #define RENEW 1
 
-krb5_error_code krb5_tgt_gen( krb5_context context, krb5_ccache ccache,
+ extern krb5_error_code krb5_tgt_gen( krb5_context context, krb5_ccache ccache,
 	krb5_principal server, krb5_data *outbuf, int opt );
 
 int client_krb5_auth( char *keytabfile, char *service, char *host,
@@ -289,11 +308,16 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 	char *sname = 0;
 	int fd = -1, len;
 	char buffer[SMALLBUFFER];
+	struct stat statb;
 
 	err[0] = 0;
-	DEBUG0( "client_krb5_auth: keytab '%s',"
+	DEBUG1( "client_krb5_auth: keytab '%s',"
 		" service '%s', host '%s', sock %d, file '%s'",
 		keytabfile, service, host, sock, file );
+	if( !safestrcasecmp(host,LOCALHOST) ){
+		host = FQDNHost_FQDN;
+		DEBUG1( "client_krb5_auth: using host='%s'", host );
+	}
 	memset((char *)&my_creds, 0, sizeof(my_creds));
 	memset((char *)&outbuf, 0, sizeof(outbuf));
 	memset((char *)&eblock, 0, sizeof(eblock));
@@ -311,6 +335,7 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 		retval = 1;
 		goto done;
 	}
+
 	if(server_principal){
 		if ((retval = krb5_parse_name(context,server_principal, &server))){
 			plp_snprintf( err, errlen, "client_krb5_auth failed - "
@@ -328,23 +353,17 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 			goto done;
 		}
 	}
-	
-	if(DEBUGL0){
-		if((retval = krb5_unparse_name(context, server, &sname))){
-			plp_snprintf( err, errlen, "client_krb5_auth failed - "
-				"krb5_unparse_name of 'server' failed: %s",
-				error_message(retval));
-			goto done;
-		}
-		logDebug( "client_krb5_auth: server '%s'", sname );
-		free(sname); sname = 0;
+
+	if((retval = krb5_unparse_name(context, server, &sname))){
+		plp_snprintf( err, errlen, "client_krb5_auth failed - "
+			"krb5_unparse_name of 'server' failed: %s",
+			error_message(retval));
+		goto done;
 	}
+	DEBUG1( "client_krb5_auth: server '%s'", sname );
 
 	my_creds.server = server;
 
-	cksum_data.data = host;
-	cksum_data.length = strlen(host);
-
 	if( keytabfile ){
 		if ((retval = krb5_sname_to_principal(context, NULL, service, 
 			 KRB5_NT_SRV_HST, &client))){
@@ -353,16 +372,14 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 			" - %s", service, error_message(retval) );
 			goto done;
 		}
-		if(DEBUGL0){
-			if((retval = krb5_unparse_name(context, client, &cname))){
-				plp_snprintf( err, errlen, "client_krb5_auth failed - "
-					"krb5_unparse_name of 'me' failed: %s",
-					error_message(retval));
-				goto done;
-			}
-			logDebug( "client_krb5_auth: client '%s'", cname );
-			free(cname); cname = 0;
+		if(cname)free(cname); cname = 0;
+		if((retval = krb5_unparse_name(context, client, &cname))){
+			plp_snprintf( err, errlen, "client_krb5_auth failed - "
+				"krb5_unparse_name of 'me' failed: %s",
+				error_message(retval));
+			goto done;
 		}
+		DEBUG1("client_krb5_auth: client '%s'", cname );
 		my_creds.client = client;
 		if((retval = krb5_kt_resolve(context, keytabfile, &keytab))){
 			plp_snprintf( err, errlen, "client_krb5_auth failed - "
@@ -414,10 +431,7 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 				plp_snprintf( err, errlen, "client_krb5_auth failed - "
 				"validating tgt"
 				" - %s", error_message(retval) );
-				DEBUG0("%s", err );
-			} else {
-				/* should be done... */
-				goto done;
+				DEBUG1("%s", err );
 			}
 		}
 
@@ -428,10 +442,7 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 				plp_snprintf( err, errlen, "client_krb5_auth failed - "
 				"renewing tgt"
 				" - %s", error_message(retval) );
-				DEBUG0("%s", err );
-			} else {
-				/* should be done... */
-				goto done;
+				DEBUG1("%s", err );
 			}
 		}
 
@@ -466,18 +477,22 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 				error_message( retval ) );
 			goto done;
 		}
-		if(DEBUGL0){
-			if((retval = krb5_unparse_name(context, client, &cname))){
-				plp_snprintf( err, errlen, "client_krb5_auth failed - "
-					"krb5_unparse_name of 'me' failed: %s",
-					error_message(retval));
-				goto done;
-			}
-			logDebug( "client_krb5_auth: client '%s'", cname );
-			free(cname); cname = 0;
+		if(cname)free(cname); cname = 0;
+		if((retval = krb5_unparse_name(context, client, &cname))){
+			plp_snprintf( err, errlen, "client_krb5_auth failed - "
+				"krb5_unparse_name of 'me' failed: %s",
+				error_message(retval));
+			goto done;
 		}
+		DEBUG1( "client_krb5_auth: client '%s'", cname );
 		my_creds.client = client;
 	}
+
+	cksum_data.data = host;
+	cksum_data.length = strlen(host);
+
+
+
 	if((retval = krb5_auth_con_init(context, &auth_context))){
 		plp_snprintf( err, errlen, "client_krb5_auth failed - "
 			"krb5_auth_con_init failed: %s",
@@ -495,7 +510,7 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 	}
   
 	retval = krb5_sendauth(context, &auth_context, (krb5_pointer) &sock,
-			   service , client, server,
+			   service, client, server,
 			   AP_OPTS_MUTUAL_REQUIRED,
 			   &cksum_data,
 			   &my_creds,
@@ -517,7 +532,7 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 		retval = 1;
 		goto done;
 	} else {
-		DEBUG0("client_krb5_auth: sequence number %d", rep_ret->seq_number );
+		DEBUG1("client_krb5_auth: sequence number %d", rep_ret->seq_number );
 	}
     /* initialize the initial vector */
     if((retval = krb5_auth_con_initivector(context, auth_context))){
@@ -533,106 +548,102 @@ int client_krb5_auth( char *keytabfile, char *service, char *host,
 	memset( &inbuf, 0, sizeof(inbuf) );
 	memset( &outbuf, 0, sizeof(outbuf) );
 
-	if( file ){
-		struct stat statb;
-
-		fd = Checkread( file, &statb );
-		if( fd < 0 ){
-			plp_snprintf( err, errlen,
-				"client_krb5_auth: could not open '%s' - '%s'", file,
-					Errormsg(errno) );
-			retval = 1;
-			goto done;
-		}
-		DEBUG0( "client_krb5_auth: opened %s, fd %d", file, fd );
-		while( (len = read( fd, buffer, sizeof(buffer)-1 )) > 0 ){
-			/* status = Write_fd_len( sock, buffer, len ); */
-			inbuf.data = buffer;
-			inbuf.length = len;
-			if((retval = krb5_mk_priv(context, auth_context, &inbuf,
-				&outbuf, NULL))){
-				plp_snprintf( err, errlen,
-					"krb5_mk_priv failed: %s",
-					error_message(retval));
-				goto done;
-			}
-			if((retval = krb5_write_message(context, (void *)&sock, &outbuf))){
-				plp_snprintf( err, errlen,
-					"krb5_write_message failed: %s",
-					error_message(retval));
-				goto done;
-			}
-			krb5_xfree(outbuf.data);
-		}
-		if( len < 0 ){
-			plp_snprintf( err, errlen,
-				"client_krb5_auth: file read failed '%s' - '%s'", file,
-					Errormsg(errno) );
-			retval = 1;
-			goto done;
-		}
-		close(fd);
-		fd = -1;
-		DEBUG0( "client_krb5_auth: file copy finished %s", file );
-		if( shutdown(sock, 1) == -1 ){
-			plp_snprintf( err, errlen,
-				"shutdown failed '%s'", Errormsg(errno) );
-			retval = 1;
-			goto done;
-		}
+	fd = Checkread( file, &statb );
+	if( fd < 0 ){
+		plp_snprintf( err, errlen,
+			"client_krb5_auth: could not open for reading '%s' - '%s'", file,
+				Errormsg(errno) );
+		retval = 1;
+		goto done;
 	}
-	/* now we fork the other direction */
-	{
-		int fds[2];
-		int pid;
-		if( pipe(fds) < 0 ){
-			plp_snprintf( err, errlen, "pipe() failed - %s",
-				Errormsg(errno) );
-			retval = 1;
+	DEBUG1( "client_krb5_auth: opened for read %s, fd %d, size %0.0f", file, fd, (double)statb.st_size );
+	while( (len = read( fd, buffer, sizeof(buffer)-1 )) > 0 ){
+		/* status = Write_fd_len( sock, buffer, len ); */
+		inbuf.data = buffer;
+		inbuf.length = len;
+		if((retval = krb5_mk_priv(context, auth_context, &inbuf,
+			&outbuf, NULL))){
+			plp_snprintf( err, errlen,
+				"krb5_mk_priv failed: %s",
+				error_message(retval));
 			goto done;
 		}
-		if( (pid = dofork(0)) < 0 ){
-			plp_snprintf( err, errlen, "dofork() failed - %s",
-				Errormsg(errno) );
-			retval = 1;
+		if((retval = krb5_write_message(context, (void *)&sock, &outbuf))){
+			plp_snprintf( err, errlen,
+				"krb5_write_message failed: %s",
+				error_message(retval));
 			goto done;
-		} else if( pid == 0 ){
-			/* we are the child */
-			close( fds[0] ); /* input side */
-			while((retval = krb5_read_message( context,&sock,&inbuf))==0){
-				if((retval = krb5_rd_priv(context, auth_context, &inbuf,
-					&outbuf, NULL))){
-					plp_snprintf( err, errlen, "dup2() failed - %s",
-						Errormsg(errno) );
-					Write_fd_str( 2, err );
-					cleanup(0);
-				}
-				if(Write_fd_len(fds[1],outbuf.data,outbuf.length) < 0) cleanup(0);
-				krb5_xfree(inbuf.data);
-				krb5_xfree(outbuf.data);
-			}
+		}
+		DEBUG4( "client_krb5_auth: freeing data");
+		krb5_xfree(outbuf.data); outbuf.data = 0;
+	}
+	if( len < 0 ){
+		plp_snprintf( err, errlen,
+			"client_krb5_auth: file read failed '%s' - '%s'", file,
+				Errormsg(errno) );
+		retval = 1;
+		goto done;
+	}
+	close(fd);
+	fd = -1;
+	DEBUG1( "client_krb5_auth: file copy finished %s", file );
+	if( shutdown(sock, 1) == -1 ){
+		plp_snprintf( err, errlen,
+			"shutdown failed '%s'", Errormsg(errno) );
+		retval = 1;
+		goto done;
+	}
+	fd = Checkwrite( file, &statb, O_WRONLY|O_TRUNC, 1, 0 );
+	if( fd < 0 ){
+		plp_snprintf( err, errlen,
+			"client_krb5_auth: could not open for writing '%s' - '%s'", file,
+				Errormsg(errno) );
+		retval = 1;
+		goto done;
+	}
+	while((retval = krb5_read_message( context,&sock,&inbuf))==0){
+		if((retval = krb5_rd_priv(context, auth_context, &inbuf,
+			&outbuf, NULL))){
+			plp_snprintf( err, errlen, "krb5_rd_priv failed - %s",
+				Errormsg(errno) );
+			Write_fd_str( 2, err );
 			cleanup(0);
-		} else {
-			close( fds[1] ); /* output side */
-			if( dup2( fds[0], sock ) == -1 ){
-				plp_snprintf( err, errlen, "dup2() failed - %s",
-					Errormsg(errno) );
-				retval = 1;
-				goto done;
-			}
-			close( fds[0] );
 		}
+		if(Write_fd_len(fd,outbuf.data,outbuf.length) < 0) cleanup(0);
+		krb5_xfree(inbuf.data); inbuf.data = 0;
+		krb5_xfree(outbuf.data); outbuf.data = 0;
 	}
-done:
+	close(fd); fd = -1;
+	fd = Checkread( file, &statb );
+	if( fd < 0 ){
+		plp_snprintf( err, errlen,
+			"client_krb5_auth: could not open for reading '%s' - '%s'", file,
+				Errormsg(errno) );
+		retval = 1;
+		goto done;
+	}
+	DEBUG1( "client_krb5_auth: reopened for read %s, fd %d, size %0.0f", file, fd, (double)statb.st_size );
+	if( dup2(fd,sock) == -1){
+		plp_snprintf( err, errlen,
+			"client_krb5_auth: dup2(%d,%d) failed - '%s'",
+			fd, sock, Errormsg(errno) );
+		retval = 1;
+	}
+	retval = 0;
+ done:
 	if( fd >= 0 ) close(fd);
+	DEBUG4( "client_krb5_auth: freeing my_creds");
 	krb5_free_cred_contents( context, &my_creds );
-	if( rep_ret )	krb5_free_ap_rep_enc_part( context, rep_ret );
-	if( err_ret )	krb5_free_error( context, err_ret );
-	if( server )	krb5_free_principal(context, server);
-	if( client )	krb5_free_principal(context, client);
+	DEBUG4( "client_krb5_auth: freeing rep_ret");
+	if( rep_ret )	krb5_free_ap_rep_enc_part( context, rep_ret ); rep_ret = 0;
+	DEBUG4( "client_krb5_auth: freeing err_ret");
+	if( err_ret )	krb5_free_error( context, err_ret ); err_ret = 0;
+	DEBUG4( "client_krb5_auth: freeing auth_context");
 	if( auth_context) krb5_auth_con_free(context, auth_context );
-	if( context )	krb5_free_context(context);
-	DEBUG0( "retval %d, error: '%s'",retval, err );
+	auth_context = 0;
+	DEBUG4( "client_krb5_auth: freeing context");
+	if( context )	krb5_free_context(context); context = 0;
+	DEBUG1( "client_krb5_auth: retval %d, error '%s'",retval, err );
 	return(retval);
 }
 
@@ -650,7 +661,7 @@ int remote_principal_krb5( char *service, char *host, char *err, int errlen )
 	int retval = 0;
 	char *cname = 0;
 
-	DEBUG0("remote_principal_krb5: service '%s', host '%s'",
+	DEBUG1("remote_principal_krb5: service '%s', host '%s'",
 		service, host );
 	if ((retval = krb5_init_context(&context))){
 		plp_snprintf( err, errlen, "%s '%s'",
@@ -669,11 +680,11 @@ int remote_principal_krb5( char *service, char *host, char *err, int errlen )
 		goto done;
 	}
 	strncpy( err, cname, errlen );
-done:
-	if( cname )		free(cname);
-	if( server )	krb5_free_principal(context, server);
-	if( context )	krb5_free_context(context);
-	DEBUG0( "remote_principal_krb5: retval %d, result: '%s'",retval, err );
+ done:
+	if( cname )		free(cname); cname = 0;
+	if( server )	krb5_free_principal(context, server); server = 0;
+	if( context )	krb5_free_context(context); context = 0;
+	DEBUG1( "remote_principal_krb5: retval %d, result: '%s'",retval, err );
 	return(retval);
 }
 
@@ -687,11 +698,8 @@ done:
 #define VALIDATE 0
 #define RENEW 1
 
-krb5_error_code krb5_tgt_gen( krb5_context context, krb5_ccache ccache,
-	krb5_principal server, krb5_data *outbuf, int opt );
-
 /* stripped down version of krb5_mk_req */
-krb5_error_code krb5_tgt_gen( krb5_context context, krb5_ccache ccache,
+ krb5_error_code krb5_tgt_gen( krb5_context context, krb5_ccache ccache,
 	 krb5_principal server, krb5_data *outbuf, int opt )
 {
 	krb5_error_code	   retval;
@@ -717,23 +725,23 @@ krb5_error_code krb5_tgt_gen( krb5_context context, krb5_ccache ccache,
 	}
 
 	/* we don't actually need to do the mk_req, just get the creds. */
-cleanup_creds:
+ cleanup_creds:
 	krb5_free_cred_contents(context, &creds);
 
-cleanup:
+ cleanup:
 
 	return retval;
 }
 
 
-char *storage;
-int nstored = 0;
-char *store_ptr;
-krb5_data desinbuf,desoutbuf;
+ char *storage;
+ int nstored = 0;
+ char *store_ptr;
+ krb5_data desinbuf,desoutbuf;
 
 #define ENCBUFFERSIZE 2*LARGEBUFFER
 
-int des_read( krb5_context context,
+ int des_read( krb5_context context,
 	krb5_encrypt_block *eblock,
 	int fd, char *buf, int len,
 	char *err, int errlen )
@@ -746,8 +754,8 @@ int des_read( krb5_context context,
 	if( len <= 0 ) return(len);
 
 	if( desinbuf.data == 0 ){
-		malloc_or_die( desinbuf.data, ENCBUFFERSIZE );
-		malloc_or_die( storage, ENCBUFFERSIZE );
+		desinbuf.data = malloc_or_die(  ENCBUFFERSIZE, __FILE__,__LINE__ );
+		storage = malloc_or_die( ENCBUFFERSIZE, __FILE__,__LINE__ );
 	}
 	if (nstored >= len) {
 		memcpy(buf, store_ptr, len);
@@ -807,17 +815,17 @@ int des_read( krb5_context context,
 }
 
 
-int des_write( krb5_context context,
+ int des_write( krb5_context context,
 	krb5_encrypt_block *eblock,
 	int fd, char *buf, int len,
 	char *err, int errlen )
 {
-	unsigned char len_buf[4];
+	char len_buf[4];
 	int cc;
 
 	if( len <= 0 ) return( len );
 	if( desoutbuf.data == 0 ){
-		malloc_or_die( desoutbuf.data, ENCBUFFERSIZE );
+		desoutbuf.data = malloc_or_die( ENCBUFFERSIZE, __FILE__,__LINE__ );
 	}
 	desoutbuf.length = krb5_encrypt_size(len, eblock->crypto_entry);
 	if (desoutbuf.length > ENCBUFFERSIZE ){
