@@ -1,3 +1,13 @@
+/***************************************************************************
+ * LPRng - An Extended Print Spooler System
+ *
+ * Copyright 1988-1999, Patrick Powell, San Diego, CA
+ *     papowell@astart.com
+ * See LICENSE for conditions of use.
+ * $Id: portable.h,v 5.3 1999/10/09 20:49:32 papowell Exp papowell $
+ ***************************************************************************/
+
+
 /***************************************************************************
  * LPRng - An Extended Print Spooler System
  *
@@ -22,18 +32,27 @@
  *    Justin Mason <jmason@iona.ie> especially.  Some of the things
  *    that you have to do to get portability are truely bizzare.
  *
- * $Id: portable.h,v 3.13 1997/12/24 20:10:12 papowell Exp $
+ * portable.h,v 3.14 1998/03/24 02:43:22 papowell Exp
  **************************************************************************/
 
 #ifndef _PLP_PORTABLE_H
 #define _PLP_PORTABLE_H 1
 
+#if !defined(EXTERN)
+#define EXTERN extern
+#define DEFINE(X) 
+#endif
+
 #ifndef __STDC__
 LPRng requires ANSI Standard C compiler
 #endif
 
 #include "config.h"
 
+#ifdef HAVE_CTYPE_H
+#include <ctype.h>
+#endif
+
 /*************************************************************************
  * ARGH: some things that "configure" can't get right.
 *************************************************************************/
@@ -123,14 +142,6 @@ LPRng requires ANSI Standard C compiler
 # define IS_LINUX
 #endif
 
-
-/*************************************************************************/
-#if defined(__hpux) || defined(_HPUX_SOURCE)
-# define IS_HPUX
-# undef _HPUX_SOURCE
-# define _HPUX_SOURCE 1
-#endif
-  
 /*************************************************************************/
 
 #if defined(__convex__) /* Convex OS 11.0 - from w_stef */
@@ -267,6 +278,9 @@ LPRng requires ANSI Standard C compiler
 
 #include <sys/stat.h>
 #include <pwd.h>
+#if defined(HAVE_SYS_SIGNAL_H)
+#  include <sys/signal.h>
+#endif
 #include <signal.h>
 #include <sys/wait.h>
 #include <ctype.h>
@@ -290,6 +304,9 @@ LPRng requires ANSI Standard C compiler
 #if !defined(HAVE_STRNCASECMP)
  int strncasecmp (const char *s1, const char *s2, int len );
 #endif
+#if !defined(HAVE_STRCASECMP_DEF)
+ int strcasecmp (const char *s1, const char *s2 );
+#endif
 
 
 /*********************************************************************
@@ -352,9 +369,9 @@ typedef struct dirent plp_dir_t;
 # include <time.h>
 #else
 # ifdef HAVE_SYS_TIME_H
-# include <sys/time.h>
-#else
-# include <time.h>
+#   include <sys/time.h>
+# else
+#   include <time.h>
 # endif
 #endif
 
@@ -608,22 +625,16 @@ XX ** NO VARARGS ** XX
  * different systems
  */
 
-#ifndef O_NONBLOCK
-#define O_NONBLOCK 0
-#endif
-
-#ifndef O_NOCTTY
-#define O_NOCTTY 0
-#endif
-
-
 #define NONBLOCK (O_NDELAY|O_NONBLOCK)
-#ifdef IS_HPUX
-#undef NONBLOCK
-#define NONBLOCK (O_NONBLOCK)
+#if defined(HPUX) && HPUX<1100
+#  undef NONBLOCK
+#  define NONBLOCK (O_NONBLOCK)
+#  undef FD_SET_FIX
+#  define FD_SET_FIX(X) (int *)
 #endif
 
 
+
 /*********************************************************************
  * AIX systems need this
  *********************************************************************/
@@ -637,13 +648,18 @@ XX ** NO VARARGS ** XX
  **********************************************************************/
 #ifdef HAVE_SIGPROCMASK
 /* a signal set */
-typedef sigset_t plp_block_mask;
+#define plp_block_mask sigset_t
 #else
 /* an integer */
-typedef int plp_block_mask;
+#define plp_block_mask int
 #endif
 
-#define FD_SET_FIX(X) X
+/**********************************************************************
+ *  Select() problems
+ **********************************************************************/
+#if !defined(FD_SET_FIX)
+# define FD_SET_FIX(X) X
+#endif
 
 /**********************************************************************
  * IPV6 and newer versions
@@ -652,7 +668,7 @@ typedef int plp_block_mask;
 int inet_pton( int family, const char *strptr, void *addr );
 #endif
 #if !defined(HAVE_INET_NTOP)
-char *inet_ntop( int family, const void *addr, char *strptr, int len );
+const char *inet_ntop( int family, const void *addr, char *strptr, size_t len );
 #endif
 
 /**********************************************************************
@@ -686,7 +702,7 @@ extern int lockf(int fd, int cmd, long size );
 /*extern int lseek(int fd, off_t pos, int how ); */
 extern int lstat(const char *path, struct stat *buf );
 #define memmove(dest,src,len) bcopy(src,dest,len)
-extern void bcopy(char *src,char *dest,int len);
+extern void bcopy(const void *src,void *dest,size_t len);
 extern int mkstemp(char *s );
 extern int openlog( const char *ident, int logopt, int facility );
 extern int perror(const char *);
@@ -704,11 +720,16 @@ extern int stat(const char *path, struct stat *buf );
 extern int strcasecmp( const char *, const char * );
 extern char *strerror( int );
 extern int strncasecmp( const char *, const char *, int n );
-extern int long strtol( char *str, char **ptr, int base );
+extern long strtol( const char *str, char **ptr, int base );
+extern double strtod( const char *str, char **ptr );
+extern int shutdown( int sock, int how );
+extern int gettimeofday(struct timeval *tp, struct timezone *tzp);
+extern int getrlimit(int resource, struct rlimit *rlp);
+extern char * sbrk(int incr);
+extern int fchmod(int fd, int mode);
 extern int strftime(char *buf, int bufsize, const char *fmt, struct tm *tm);
 extern void syslog(int, const char *, ...);
 extern int system( const char *str );
-extern int tgetent( char *buffer, const char *name );
 extern time_t time( time_t *t );
 extern int tolower( int );
 extern int toupper( int );
@@ -733,21 +754,14 @@ extern int getdtablesize(void);
 #endif
 #endif
 
-#ifdef HPUX
+#if !defined(HAVE_SYSLOG_DEF)
 extern void syslog(int, const char *, ...);
+#endif
 #if !defined(HAVE_OPENLOG_DEF)
 extern int openlog( const char *ident, int logopt, int facility );
 #endif
-#undef FD_SET_FIX
-#define FD_SET_FIX(X) (int *)
-#endif
 
 #ifdef IS_AIX32
-extern int tgetent(char *, char *);
-#if !defined(HAVE_OPENLOG_DEF)
-void openlog(const char *, int, int);
-void syslog(int, const char *, ...);
-#endif
 extern int seteuid(uid_t);
 #endif
 
@@ -784,4 +798,27 @@ struct sockaddr_in6 {
 # include <resolv.h>
 #endif
 
+/* the dreaded QUAD_T strikes again... */
+#if defined(quad_t) && qaud_t == NONE
+# undef HAVE_QUAD_T
+#else
+# define HAVE_QUAD_T 1
+/* suspender and belts on this one */
+struct have_quad_t {
+	quad_t t;
+};
+#endif
+
+#ifdef HAVE_INNETGR
+#if !defined(HAVE_INNETGR_DEF)
+extern int innetgr(const char *netgroup,
+    const char *machine, const char *user, const char *domain);
+#endif
+#endif
+
+
+#define Cast_int_to_voidstar(v) ((void *)(long)(v))
+#define Cast_ptr_to_int(v) ((int)(long)(v))
+#define Cast_ptr_to_long(v) ((long)(v))
+
 #endif	/* PLP_PORTABLE_H */
